#include "stdafx.h"
#include "LottoManCE.h"
#include "LottoManCEDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



CMember::CMember()
{
	m_bMarkedForDeletion = FALSE;
  m_nNumbers[0] = 0;
  m_nNumbers[1] = 0;
  m_nNumbers[2] = 0;
  m_nNumbers[3] = 0;
  m_nNumbers[4] = 0;
  m_nNumbers[5] = 0;
  m_bEnabled = TRUE;
}

CMember::CMember(const CMember& member)
{
	*this = member;
}

CMember::~CMember()
{
}

CMember& CMember::operator=(const CMember& member)
{
	m_sName = member.m_sName;
  for (int i=0; i<6; i++)
    m_nNumbers[i] = member.m_nNumbers[i];
  m_bEnabled = member.m_bEnabled;

	m_bMarkedForDeletion = member.m_bMarkedForDeletion;

	return *this;
}

void CMember::Serialize(CArchive& ar)
{
  //let the parent do its stuff
  CObject::Serialize(ar);

  ASSERT(!m_bMarkedForDeletion);

  //Serialize the elements
  if (ar.IsLoading())
  {
    WORD wConfigVersion;
    ar >> wConfigVersion;
    ar >> m_sName;
		for (int i=0; i<6; i++)
			ar >> m_nNumbers[i];

    if (wConfigVersion > 0x100)
      ar >> m_bEnabled;
    else
      m_bEnabled = TRUE;
  }
  else
  {
    WORD wConfigVersion = 0x101;
    ar << wConfigVersion;
    ar << m_sName;
		for (int i=0; i<6; i++)
			ar << m_nNumbers[i];

    ar << m_bEnabled;
  }
}





IMPLEMENT_DYNCREATE(CLottoManDoc, CDocument)

BEGIN_MESSAGE_MAP(CLottoManDoc, CDocument)
	//{{AFX_MSG_MAP(CLottoManDoc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CLottoManDoc::CLottoManDoc()
{
  m_nListViewStyle = 2;
}

CLottoManDoc::~CLottoManDoc()
{
  FreeAllMemory();
}

BOOL CLottoManDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	return TRUE;
}

void CLottoManDoc::Serialize(CArchive& ar)
{
  if (ar.IsLoading())
  {
    FreeAllMemory();

    //Version Stamp
    WORD wConfigVersion;
    ar >> wConfigVersion;

    //the members
    int nMembers;
    ar >> nMembers;
    m_Members.SetSize(nMembers);
    for (int i=0; i<nMembers; i++)
    {
      CMember* pMember = new CMember;
      pMember->Serialize(ar);
      m_Members.SetAt(i, pMember);
    }

    //The list view style
    if (wConfigVersion > 0x100)
      ar >> m_nListViewStyle;
  }
  else
  {
    //Version Stamp
    WORD wConfigVersion = 0x101;
    ar << wConfigVersion;

    //The date ranges
    ar << NumberOfMembers();
    for (int i=0; i<m_Members.GetSize(); i++)
    {
      CMember* pMember = m_Members.GetAt(i);
      if (!pMember->m_bMarkedForDeletion)
        pMember->Serialize(ar);
    }

    //The list view style
    ar << m_nListViewStyle;
  }
}

#ifdef _DEBUG
void CLottoManDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CLottoManDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

void CLottoManDoc::FreeAllMemory()
{
  for (int i=0; i<m_Members.GetSize(); i++)
    delete m_Members.GetAt(i);
  m_Members.RemoveAll();
}

void CLottoManDoc::MarkMemberAsDeleted(int nIndex)
{
  CMember* pMember = m_Members.GetAt(nIndex);
  pMember->m_bMarkedForDeletion = TRUE;
}

int CLottoManDoc::NumberOfMembers() const
{
  int nMembers = 0;
  for (int i=0; i<m_Members.GetSize(); i++)
  {
    CMember* pMember = m_Members.GetAt(i);
    if (!pMember->m_bMarkedForDeletion)
      nMembers++;
  }
  return nMembers;
}

CString CLottoManDoc::CreateUniqueMemberName() const
{
  CString sName;
  BOOL bExist = TRUE;
  int nNumber = (m_Members.GetSize() + 1);
  while (bExist)
  {
    CString sNum;
    sNum.Format(_T("%d"), nNumber);
    AfxFormatString1(sName, IDS_NEW_MEMBER, sNum);
    bExist = DoesMemberNameExist(sName, -1);
    nNumber++;
  }

  return sName;
}

BOOL CLottoManDoc::DoesMemberNameExist(const CString& sName, int nExistingIndex) const
{
  for (int i=0; i<m_Members.GetSize(); i++)
  {
    CMember* pMember = m_Members.GetAt(i);
    if (!pMember->m_bMarkedForDeletion && (i != nExistingIndex) && (pMember->m_sName.CompareNoCase(sName) == 0))
      return TRUE;
  }
  return FALSE;
}

CMember* CLottoManDoc::GetAtMember(int nMemberNum) const
{
  int nValidMembers = -1;
  int nOffset = 0;
  while (nValidMembers < nMemberNum)
  {
    if (m_Members.GetAt(nOffset)->m_bMarkedForDeletion)
      nOffset++;
    else
    {
      nValidMembers++;
      nOffset++;
    }
  }

  return m_Members.GetAt(nOffset-1);
}



void CLottoManDoc::DeleteContents() 
{
	FreeAllMemory();
	
	CDocument::DeleteContents();
}
